<?php
/**
 * Zen
 *
 * @author    Zen
 * @copyright Copyright (c) 2022 Zen
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License ("AFL") v. 3.0
 *
 * https://www.zen.com/
 */

use Zen\Payment\Notification;
use Zen\Payment\Util;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class ZenNotificationModuleFrontController
 *
 * @property bool display_column_left
 * @property bool display_footer
 * @property bool display_column_right
 * @property bool display_header
 */
class ZencomNotificationModuleFrontController extends ModuleFrontController
{

    /**
     * Class init
     */
    public function init()
    {
        $this->display_column_left = false;
        $this->display_column_right = false;
        $this->display_footer = false;
        $this->display_header = false;
        parent::init();
    }

    /**
     * verify signature, json, create signature and change status if its okay
     */
    public function process()
    {

        $notification = new Notification(
            Configuration::get('ZEN_IPN_SECRET')
        );

        // it can be order data or notification code - depends on verification notification
        $resultCheckRequestNotification = $notification->checkRequest();

        if (is_int($resultCheckRequestNotification)) {
            echo Notification::formatResponse(Notification::NS_ERROR);
            exit();
        }

        if ($resultCheckRequestNotification['type'] === Notification::TRT_REFUND) {
            if ($resultCheckRequestNotification['status'] !== Notification::STATUS_ACCEPTED) {
                echo Notification::formatResponse(Notification::NS_OK);
                exit();
            }

            $order = new Order(
                Util::extractMerchantTransactionId($resultCheckRequestNotification['merchantTransactionId'])
            );

            if (!($orderId = $order->id)) {
                echo Notification::formatResponse(Notification::NS_ERROR);
                exit();
            }

            $orderCurrentState = $order->getCurrentState();

            $stateArray = [
                _PS_OS_CANCELED_,
                _PS_OS_REFUND_,
            ];

            if (in_array($orderCurrentState, $stateArray)) {
                echo Notification::formatResponse(Notification::NS_ERROR);
                exit();
            }

            $orderId = (int)$orderId;

            $history = new OrderHistory();
            $history->id_order = $orderId;
            $history->changeIdOrderState(Configuration::get('PS_OS_REFUND'), $orderId);
            $history->addWithemail(true, array());

            echo Notification::formatResponse(Notification::NS_OK);
            exit;
        }

        $order = new Order($resultCheckRequestNotification['merchantTransactionId']);
        $orderId = $order->id;

        if (!$orderId) {
            echo Notification::formatResponse(Notification::NS_ERROR);
            exit();
        }

        $lastOrderState = $order->getCurrentState();
        if ($lastOrderState === _PS_OS_PAYMENT_) {
            echo Notification::formatResponse(Notification::NS_ERROR);
            exit();
        }

        $currencyInfo = Currency::getCurrency($order->id_currency);

        if (!$notification::checkRequestAmount(
            $resultCheckRequestNotification,
            $order->total_paid,
            $currencyInfo["iso_code"]
        )) {
            echo Notification::formatResponse(Notification::NS_ERROR);
            exit();
        }

        switch ($resultCheckRequestNotification['status']) {
            case Notification::STATUS_ACCEPTED:
                $notificationTransactionStatus = _PS_OS_PAYMENT_;
                break;
            case Notification::STATUS_REJECTED:
                $notificationTransactionStatus = _PS_OS_ERROR_;
                break;
            default:
                echo Notification::formatResponse(Notification::NS_OK);
                exit;
        }

        $history = new OrderHistory();
        $history->id_order = (int)$orderId;

        // Change order state
        $history->changeIdOrderState($notificationTransactionStatus, (int)$orderId);
        $history->addWithemail(true);

        if (_PS_OS_PAYMENT_ === $notificationTransactionStatus) {
            $payments = $order->getOrderPaymentCollection()->getResults();

            if (count($payments) > 0) {
                $payments[0]->transaction_id = $resultCheckRequestNotification['transactionId'];
                $payments[0]->update();
            }
        }
        echo Notification::formatResponse(Notification::NS_OK);
        exit();
    }
}
